		title   MZLOAD - Datei im MZ-Format laden

; ===========================================================================
; Programm zum Laden von Dateien, die im MZ-Format auf der Datassette
; vorliegen. Der Aufruf erfolgt durch
;
;   MZLOAD optionen dosfilename "tapefilename"
;
; Es wird die naechste auf der Cassette vorliegende Datei mit dem Namen
; tapefilename oder, falls tapefilename nicht angegeben wird, die naechste
; vorliegende Datei ueberhaupt eingeladen und in der Datei mit dem Namen 
; dosfilename im M12-Format, also mit dem Tapeheader vorneweg, abgelegt. 
; Alle Parameter sind optional. Fuer Einzelheiten siehe den Hilfetext.
;
; Autor: Andreas Jung, Karl-Heinz Mau
; Datum: 28.11.1998
; ===========================================================================

main            segment

; ===========================================================================
; Registerbelegung in den kritischen Prozeduren:
; ---------------------------------------------------------------------------
; AX    Akku, Kommunikation mit den Ports
; BL    gelesenes Byte
; BH    Bitzaehler
; CX    Zaehler fuer den Countdown und die eingelesenen Bytes
; DX    Portadresse
; SI    Pruefsumme
; DI    Zieladresse beim Einlesen
; ===========================================================================
		
		assume  cs:main, ds:main, es:main, ss:main

; ===========================================================================
; Konstanten und Portadressen
; ===========================================================================

portb           equ     61h             ; Port B
prndata         equ     0378h           ; Datenregister des Druckerports
prnctrl         equ     0379h           ; Steuerregister des Druckerports
counter         equ     042h            ; Zaehler
cntctrl         equ     043h            ; Steuerregister des Zaehlers
pulse           equ     579             ; mittlere Dauer eines Impulses

; ===========================================================================
; Macro startcnt startet den Zaehler neu
; ===========================================================================

startcnt        macro
		mov     al, 10111000b   ; 16 Bit laden, Modus 4
		out     cntctrl, al     ; Steuerwort schreiben
		mov     al, 0           ; Zaehler an den Anfang setzen
		out     counter, al     ; Lo-Byte schreiben
		out     counter, al     ; Hi-Byte schreiben
		endm

; ===========================================================================
; Macro readcnt liest den aktuellen Zaehlerstand in den Akkumulator aus.
; ===========================================================================

readcnt         macro
		mov     al, 10000000b   ; Zaehler-Latch-Befehl
		out     cntctrl, al     ; Steuerwort schreiben
		in      al, counter     ; Lo-Byte lesen
		mov     ah, al          ; und retten
		in      al, counter     ; Hi-Byte lesen
		xchg    ah, al          ; und merken
		endm

; ===========================================================================
; Macro print gibt die $-terminierte Zeichenkette ab der Adresse, die durch
; das Argument gegeben ist, auf dem Bildschirm aus.
; ===========================================================================

print           macro   adr
		mov     ah, 09h         ; DOS-Funktion 09h
		lea     dx, adr         ; Adresse der Zeichenkette
		int     21h             ; Zeichenkette ausgeben
		endm

; ===========================================================================
; Adressen im PSP
; ===========================================================================

		org     80h
arglen          db      ?               ; Laenge der Kommandozeile
args            db      7fh dup (?)     ; Kommandozeile

; ===========================================================================
; Beginn des Programmcodes
; ===========================================================================

		org     100h            ; Start des .COM-Programmes
start:          jmp     mainprog        ; an den Anfang springen

; ===========================================================================
; Diverse Texte
; ===========================================================================

info:
db "Ŀ", 0dh, 0ah
db " MZLOAD V0.2 - MZ700 Loader - (C) 1998 by Andreas Jung and Karl-Heinz Mau ", 0dh, 0ah
db "", 0dh, 0ah
db 0dh, 0ah
db 'Usage: MZLOAD options dosfilename "tapefilename"', 0dh, 0ah
db 0dh, 0ah
db "Loads the next MZ700 file called tapefilename from the datassette and", 0dh, 0ah 
db "stores it in M12 format in a disk file called dosfilename. All parameters", 0dh, 0ah
db "are optional. If tapefilename is omitted, then the next file on tape is", 0dh, 0ah
db "loaded. If dosfilename is omitted, then the file is stored under a free", 0dh, 0ah
db "name that is similar to the tape name of the file.", 0dh, 0ah
db 0dh, 0ah
db "The following options can be specified:", 0dh, 0ah
db 0dh, 0ah
db "        -x        invert polarity when reading", 0dh, 0ah
db "        -s        read first copy, don't verify", 0dh, 0ah
db 0dh, 0ah
db "The program can be aborted by pressing the STOP button on the datassette.", 0dh, 0ah
db "Run MZLOAD only under plain DOS, not under Windows, because during loading,", 0dh, 0ah 
db "all interrupts are disabled.", 0dh, 0ah
db "$"

press           db      "Press play on tape.", 0dh, 0ah, "$"
ok              db      "ok", 0dh, 0ah, "$"
searching       db      "Searching ", "$"
eol             db      0dh, 0ah, "$"
found           db      "Found ", "$"
hdvererr        db      "Verify error, using first header copy.", 0dh, 0ah, "$"
hdusesnd        db      "Corrupt first header copy, using second one.", 0dh, 0ah, "$"
hdcorrupt       db      "Found corrupt header block.", 0dh, 0ah, "$"
toobig          db      "File is too big.", 0dh, 0ah, "$"
loading         db      "Loading", 0dh, 0ah, "$"
datvererr       db      "Verify error, using first data copy.", 0dh, 0ah, "$"
datusesnd       db      "Corrupt first data copy, using second one.", 0dh, 0ah, "$"
datcorrupt      db      "Data is corrupted.", 0dh, 0ah, "$"
ready           db      "Ready.", 0dh, 0ah, "$"
lderror         db      "Load error.", 0dh, 0ah, "$"
saving          db      "Saving as ", "$"
sverror         db      "Disk write error.", 0dh, 0ah, "$"
panic           db      "Program aborted.", 0dh, 0ah, "$"
noname          db      "Cannot create free name for file.", 0dh, 0ah, "$"
defname         db      "MZFILE"        ; Defaultname zum Abspeichern
defnlen         equ     6               ; Laenge des Defaultnamen
defext          db      ".M12"          ; Defaultextension zum Abspeichern
defxlen         equ     4               ; Laenge der Defaultextension

; ===========================================================================
; Umwandlungstabelle SHARPSCII -> ASCII
; ===========================================================================

shasctab        db      000h, 001h, 002h, 003h, 004h, 005h, 006h, 007h
		db      008h, 009h, 00ah, 00bh, 00ch, 00dh, 00eh, 00fh
		db      010h, 011h, 012h, 013h, 014h, 015h, 016h, 017h
		db      018h, 019h, 01ah, 01bh, 01ch, 01dh, 01eh, 01fh
		db      " !", '"', "#$%&'()*+,-./"
		db      "0123456789:;<=>?"
		db      "@ABCDEFGHIJKLMNO"
		db      "PQRSTUVWXYZ[\]^_"
		db      "?????????~??????"
		db      "\\//?"
		db      "}//\\\\\//^\\//"
		db      "_#e`~tgh/bxdrpc"
		db      "qazwsui͙kfvj"
		db      "n/m//\ol\y{\"
		db      "ĳ"
		db      "´ڿ"
		db      "?ĳ\/"
		db      "??XO?ۜ?"

; ===========================================================================
; Prozedur initcnt initialisiert den Zaehler
; ===========================================================================

initcnt         proc    near
		in      al, portb
		and     al, 11111101b   ; Zaehler vom Lautsprecher abkoppeln (bit1 aus)
		or      al, 00000001b   ; Zaehler aktivieren (bit0 an)
		out     portb, al
		startcnt                ; Zaehler starten
		ret
initcnt         endp

; ===========================================================================
; Prozedur waiting wartet 2048*CX Timerzyklen
; ===========================================================================

waiting         proc    near
waitloop:       startcnt                ; Zaehler starten
wcntloop:       readcnt                 ; Zaehler auslesen
		cmp     ax, -2048       ; aktuellen Zaehlerstand vergleichen
		ja      wcntloop        ; bis zum Ablauf warten
		loop    waitloop        ; das ganze CX-mal
		ret
waiting         endp

; ===========================================================================
; Prozedur abort verlaesst das Programm panikartig
; ===========================================================================

abort           proc    near
		sti                     ; Interrupts zulassen
		call    motoroff        ; Motor ausschalten
		print   panic           ; Abbruchmeldung ausgeben
		mov     ax, 4c01h       ; Programm mit Fehler beenden
		int     21h             ; DOS aufrufen
abort           endp
							   
; ===========================================================================
; Prozedur readbit liest ein Bit vom Band in das Carry-Flag ein
; ===========================================================================

readbit         proc    near
		mov     dx, prnctrl     ; Steuerregister des Druckerports
		cmp     inverse, 0      ; falls Invers-Option nicht gesetzt
		je      loloop          ; dann nicht-invertiert lesen
loloopi:        in      al, dx          ; Ende des Lo-Pegels abwarten
		test    al, 20h         ; falls PLAY-Taste nicht gedrueckt
		jnz     abort           ; dann Programm abbrechen
		test    al, 40h         ; falls immer noch Lo-Pegel
		jnz     loloopi         ; dann warten
hiloopi:        in      al, dx          ; Ende des Hi-Pegels abwarten
		test    al, 20h         ; falls PLAY-Taste nicht gedrueckt
		jnz     abort           ; dann Programm abbrechen
		test    al, 40h         ; falls immer noch Hi-Pegel
		jz      hiloopi         ; dann warten
		readcnt                 ; Zaehler auslesen
		mov     dx, ax          ; und retten
		startcnt                ; und neu starten
		jmp     rbvalue         ; zur Auswertung verzweigen
loloop:         in      al, dx          ; Ende des Lo-Pegels abwarten
		test    al, 20h         ; falls PLAY-Taste nicht gedrueckt
		jnz     abort           ; dann Programm abbrechen
		test    al, 40h         ; falls immer noch Lo-Pegel
		jz      loloop          ; dann warten
hiloop:         in      al, dx          ; Ende des Hi-Pegels abwarten
		test    al, 20h         ; falls PLAY-Taste nicht gedrueckt
		jnz     abort           ; dann Programm abbrechen
		test    al, 40h         ; falls immer noch Hi-Pegel
		jnz      hiloop         ; dann warten
		readcnt                 ; Zaehler auslesen
		mov     dx, ax          ; und retten
		startcnt                ; und neu starten
rbvalue:        cmp     dx, -pulse      ; mit Schwellwert vergleichen
		jae     rbnull          ; groesser oder gleich, dann Null
		stc                     ; sonst Eins, Carry setzen
		ret                     ; und fertig
rbnull:         clc                     ; Null, dann kein Carry
		ret                     ; und fertig
readbit         endp

; ===========================================================================
; Prozedur readbyte liest ein Byte vom Band in das Register BL ein und
; aktualisiert die Pruefsumme in SI. Im Fehlerfall wird ein gesetztes
; Carry-Flag zurueckgegeben.
; Der zusaetzliche Einsprungpunkt plainbyte dient dazu, ein Byte ohne einen
; einleitenden langen Impuls zu lesen.
; ===========================================================================

readbyte        proc    near
		call    readbit         ; Einleitungsimpuls holen
; >>>>>>>>>> fr test diesen befehl raus
;		jnc     rberror         ; muss langer Impuls sein
plainbyte:      mov     bh, 8           ; insgesamt 8 Bit holen
rbloop:         call    readbit         ; ein Bit holen
		jnc     rbzero          ; falls Einsbit:
		inc     si              ; dann Pruefsumme erhoehen
rbzero:         rcl     bl, 1           ; Bit reinschieben
		dec     bh              ; naechstes Bit anvisieren
		jnz     rbloop          ; und weiterlesen
		clc                     ; Erfolg melden
		ret
rberror:        mov     bl, 0           ; im Fehlerfall ein Nullbyte zurueck
		stc                     ; und Fehler melden
		ret
readbyte        endp

; ===========================================================================
; Prozedur sync sucht eine Synchronisation, d.h. eine Folge von mindestens
; neun kurzen Impulsen und einem abschliessenden langen Impuls.
; ===========================================================================

sync            proc    near
syncsrch:       call    readbit         ; ueber lange Impulse hinweggehen
		jc      syncsrch
		mov     cx, 9           ; mindestens neun kurze Impulse
synctest:       call    readbit         ; Impuls holen
		dec     cx              ; vorigen kurzen Impuls zaehlen
		jc      syncsrch        ; langer Impuls, dann von vorne
		jnz     synctest        ; noch keine neun kurzen, dann weiter
syncover:       call    readbit         ; weitere kurze Impulse ueberlesen
		jnc     syncover
		ret                     ; nach erstem langen Impuls fertig
sync            endp

; ===========================================================================
; Prozedur readgap sucht die naechste Synchronisation in Form eines Gaps 
; (mindestens neun kurze Impulse) mit anschliessendem Tapemark der Laenge BX 
; (BX lange Impulse, BX kurze Impulse, ein langer Impuls).
; ===========================================================================

readgap         proc    near
rgsync:         call    sync            ; Synchronisation suchen
rgtm:           mov     cx, 0           ; Impulszaehler initialisieren
rgtm1:          call    readbit         ; Impuls lesen
		inc     cx              ; vorigen langen Impuls zaehlen
		jc      rgtm1           ; weiter solange Impulse lang sind
		cmp     cx, bx          ; mit gewuenschter Anzahl vergleichen
		jne     rgsync          ; neu synchronisieren falls ungleich
		mov     cx, 0           ; Impulszaehler initialisieren
rgtm0:          call    readbit         ; Impuls lesen
		inc     cx              ; vorigen kurzen Impuls zaehlen
		jnc     rgtm0           ; weiter solange Impulse kurz sind
		cmp     cx, bx          ; mit gewuenschter Anzahl vergleichen
		je      rgok            ; fertig falls gleich
		cmp     cx, 9           ; sonst testen ob wenigstens 9 kurze
		jb      rgsync          ; nein, dann neu synchronisieren
		jmp     rgtm            ; sonst als Sync gelten lassen
rgok:           ret                     ; langer Impuls wurde bereits gelesen
readgap         endp

; ===========================================================================
; Prozedur readblk liest einen Block von CX Bytes an die Adresse DI. Die sich
; an den Block anschliessende Pruefsumme wird ebenfalls gelesen und mit dem
; Sollwert verglichen. Im Fehlerfall wird ein gesetztes Carry-Flag zurueck-
; gegeben.
; Der Einsprungpunkt  geht davon aus, dass der einleitende lange
; Impuls des ersten Bytes bereits gelesen wurde.
; ===========================================================================

readblk         proc    near
		call    readbit         ; einleitenden langen Impuls holen
		jnc     rdblkerr        ; kein langer Impuls, dann Fehler
plainblk:       mov     si, 0           ; Pruefsumme initialisieren
		call    plainbyte       ; erstes Byte holen
		cmp     cx, 0           ; falls kein Byte zu lesen
		je      rdblkchk        ; dann gleich zur Checksumme

		; Programmiertrick: Falls verzweigt wird, ist CX bereits 
		; Null. Ein Retten der Pruefsumme nach CX im Falle des 
		; leeren Blockes ist also nicht noetig.

rdblkloop:      jc      rdblkerr        ; eventuell Fehler beruecksichtigen
		mov     [di], bl        ; gelesenes Byte ablegen
		inc     di              ; naechstes Byte anvisieren
		dec     cx
		jz      rdblkexit       ; und aussteigen wenn fertig
		call    readbyte        ; ein Byte lesen
		jmp     rdblkloop       ; und zum Anfang der Leseschleife
rdblkexit:      mov     cx, si          ; Pruefsumme retten
		call    readbyte        ; Hi-Byte der Pruefsumme holen
rdblkchk:       jc      rdblkerr        ; eventuell Fehler beruecksichtigen
		sub     ch, bl          ; mit Sollwert vergleichen
		call    readbyte        ; Lo-Byte der Pruefsumme holen
		jc      rdblkerr        ; eventuell Fehler beruecksichtigen
		sub     cl, bl          ; mit Sollwert vergleichen
		cmp     cx, 0           ; Vergleichswert muss Null sein
;>>>>>>>>>>>>>>>>> fr testzwecke folgender befehl raus
;		jne     rdblkerr        ; sonst Fehler
		clc                     ; Erfolg melden
		ret
rdblkerr:       stc                     ; Fehler melden
		ret
readblk         endp

; ===========================================================================
; Prozedur verify vergleicht den Block der Laenge CX ab Adresse DI mit den
; hereinkommenden Daten inklusive Pruefsumme. Bei einem Verifikationsfehler 
; wird ein gesetztes Carry-Flag zurueckgegeben. Die Prozedur geht davon aus, 
; dass der lange Impuls des ersten Bytes bereits gelesen wurde.
; ===========================================================================

verify          proc    near
		mov     ax, 0           ; Anzahl Abweichungen
		mov     diff, ax        ; initialisieren
		mov     si, 0           ; Pruefsumme initialisieren
		call    plainbyte       ; erstes Byte holen
		cmp     cx, 0           ; falls kein Byte zu lesen
		je      verchk          ; dann gleich zur Checksumme

		; Programmiertrick: Falls verzweigt wird, ist CX bereits 
		; Null. Ein Retten der Pruefsumme nach CX im Falle des 
		; leeren Blockes ist also nicht noetig.

verloop:        jc      vererr          ; eventuell Fehler beruecksichtigen
		cmp     [di], bl        ; gelesenes Byte vergleichen
		je      verok           ; verzweigen falls gleich
		inc     diff            ; sonst Anzahl Abweichungen erhoehen
verok:          inc     di              ; naechstes Byte anvisieren
		dec     cx
		jz      verexit         ; und aussteigen wenn fertig
		call    readbyte        ; ein Byte lesen
		jmp     verloop         ; und zum Anfang der Leseschleife
verexit:        mov     cx, si          ; Pruefsumme retten
		call    readbyte        ; Hi-Byte der Pruefsumme holen
verchk:         jc      vererr          ; eventuell Fehler beruecksichtigen
		sub     ch, bl          ; mit Sollwert vergleichen
		call    readbyte        ; Lo-Byte der Pruefsumme holen
		jc      vererr          ; eventuell Fehler beruecksichtigen
		sub     cl, bl          ; mit Sollwert vergleichen
		cmp     cx, 0           ; Vergleichswert muss Null sein
		jne     vererr          ; sonst Fehler
		mov     ax, diff        ; Anzahl Abweichungen holen
		cmp     ax, 0           ; falls ungleich Null
		jne     vererr          ; dann Verifikationsfehler melden
		clc                     ; sonst Erfolg melden
		ret
vererr:         stc                     ; Fehler melden
		ret
verify          endp

; ===========================================================================
; Prozedur sprint gibt den String ab Adresse SI der Laenge CX auf dem
; Bildschirm aus.
; ===========================================================================

sprint          proc    near
		cmp     cx, 0           ; falls Laenge gleich null
		je      spready         ; dann nichts ausgeben
sploop:         mov     dl, [si]        ; Zeichen holen
		mov     ah, 02h
		int     21h             ; Zeichen ausgeben
		inc     si              ; naechstes Zeichen anvisieren
		loop    sploop          ; und weitermachen
spready:        ret
sprint          endp

; ===========================================================================
; Prozedur prepargs bearbeitet die Kommandozeile. Es werden der Name der 
; DOS-Datei und der Name der gesuchten Cassettendatei sowie gegebenenfalls 
; weitere Kommandozeilenparameter gesucht. Ist die Kommandozeile fehlerhaft,
; so wird ein gesetztes Carry-Flag zurueckgegeben.
; ===========================================================================

prepargs        proc    near
		mov     ax, 0           ; Initialisierungen vornehmen
		mov     dosname, ax     ; Name der DOS-Datei
		mov     dnlen, ax       ; Laenge des Namen
		mov     tapename, ax    ; Name der Cassettendatei
		mov     tnlen, ax       ; Laenge des Namen
		mov     inverse, al     ; Invers-Option
		mov     single, al      ; Single-Option
		lea     si, args        ; Kommandozeile absuchen
		mov     dl, arglen      ; Laenge der Kommandozeile
		mov     dh, 0           ; nach DX
		add     dx, si          ; plus Anfang = Ende
paloop:         cmp     si, dx          ; falls Kommandozeilenende
		jb      paget
		jmp     paready         ; dann fertig
paget:          mov     al, [si]        ; Zeichen holen
		cmp     al, " "         ; falls Blank
		jne     noblank
		inc     si              ; dann ueberlesen
		jmp     paloop          ; und weitersuchen
noblank:        cmp     al, "-"         ; falls Minuszeichen
		jne     nominus
		inc     si              ; auf erstes Optionszeichen zeigen
		cmp     si, dx          ; falls Kommandozeilenende
		jae     paerror         ; dann Fehler melden
		mov     al, [si]        ; erstes Optionszeichen holen
		cmp     al, " "         ; falls keines vorhanden
		je      paerror         ; dann Fehler melden
optloop:        cmp     al, "x"         ; falls Option -x (invertieren)
		jne     noxopt
		mov     al, inverse     ; falls Option schon angegeben
		cmp     al, 0
		jne     paerror         ; dann Fehler melden
		mov     inverse, 1      ; sonst Option eintragen
optnext:        inc     si              ; naechstes Optionszeichen anvisieren
		cmp     si, dx          ; falls Kommandozeilenende
		jae     paready         ; dann fertig
		mov     al, [si]        ; sonst Optionszeichen holen
		cmp     al, " "         ; falls Blank
		je      paloop          ; dann naechsten Parameter
		jmp     optloop         ; sonst naechste Option
noxopt:         cmp     al, "s"         ; falls nicht Option -s (single copy)
		jne     paerror         ; dann unerlaubte Option
		mov     al, single      ; falls Option schon angegeben
		cmp     al, 0
		jne     paerror         ; dann Fehler melden
		mov     single, 1       ; sonst Option eintragen
		jmp     optnext         ; und naechst Option bearbeiten
paerror:        stc                     ; bei Fehler Misserfolg melden
		ret
nominus:        cmp     al, '"'         ; falls Anfuehrungszeichen
		jne     noquote
		mov     ax, tapename    ; dann Tape-Namen holen
		cmp     ax, 0           ; falls Tape-Name schon da
		jne     paerror         ; dann Fehler
		inc     si              ; auf erstes Namenzeichen zeigen
		mov     tapename, si    ; Namensanfang ablegen
		mov     cx, 0           ; Zaehler fuer die Namenlaenge
tnloop:         cmp     si, dx          ; falls Kommandozeilenende
		jae     paerror         ; dann Fehler
		mov     al, [si]        ; Zeichen holen
		cmp     al, '"'         ; falls Anfuehrungszeichen
		je      quotstop        ; dann fertig
		inc     cx              ; sonst Laenge erhoehen
		inc     si              ; naechstes Zeichen anvisieren
		jmp     tnloop          ; und weiterlesen
quotstop:       mov     tnlen, cx       ; Laenge des Tape-Namen ablegen
		inc     si              ; naechstes Zeichen anvisieren
		cmp     si, dx          ; falls Kommandozeilenende
		jae     paready         ; dann fertig
		mov     al, [si]        ; sonst Zeichen holen
		cmp     al, " "         ; es muss ein Blank sein
		jne     paerror         ; wenn nicht, dann Fehler
		jmp     paloop          ; ansonsten weiterlesen
noquote:        mov     ax, dosname     ; DOS-Namen holen
		cmp     ax, 0           ; falls DOS-Name schon da
		jne     paerror         ; dann Fehler
		mov     dosname, si     ; sonst DOS-Name gefunden
		mov     cx, 0           ; Laenge zaehlen
dnloop:         inc     cx              ; Laenge erhoehen
		inc     si              ; naechstes Zeichen anvisieren
		cmp     si, dx          ; falls Kommandozeilenende
		jae     dnstop          ; dann Name fertig
		mov     al, [si]        ; sonst Zeichen holen
		cmp     al, " "         ; falls Blank
		je      dnstop          ; dann Name fertig
		cmp     al, '"'         ; falls Anfuehrungszeichen
		je      paerror         ; dann Fehler
		jmp     dnloop          ; sonst weiterlesen
dnstop:         mov     dnlen, cx       ; Namenlaenge ablegen
		mov     al, 0           ; Namen mit 0-Byte abschliessen
		mov     [si], al
		inc     si              ; und dahinter aufsetzen
		jmp     paloop          ; zur Suchschleife zurueck
paready:        clc                     ; Erfolg melden
		ret
prepargs        endp                            

; ===========================================================================
; Prozedur motoron schaltet den Motor ein und wartet die Anlaufzeit ab.
; ===========================================================================

motoron         proc    near
		cli                     ; Interrupts sperren
		mov     dx, prndata     ; Motor einschalten
		mov     al, 80h
		out     dx, al
		call    initcnt         ; Zaehler initialisieren
		mov     cx, 244         ; halbe Sekunde warten
		call    waiting
		sti
		ret
motoron         endp

; ===========================================================================
; Prozedur motoroff schaltet den Motor aus.
; ===========================================================================

motoroff        proc    near
		mov     dx, prndata     ; Motor ausschalten
		mov     al, 00h
		out     dx, al
		ret
motoroff        endp

; ===========================================================================
; Prozedur playkey ueberprueft den Zustand der PLAY-Taste und fordert
; gegebenenfalls dazu auf, diese zu druecken.
; ===========================================================================

playkey         proc    near
		mov     dx, prnctrl     ; PLAY-Taste abfragen
		in      al, dx
		test    al, 20h
		jz      playok          ; falls gedrueckt, dann ok
		print   press           ; sonst "press play on tape" ausgeben
		mov     dx, prnctrl     ; PLAY-Taste erneut abfragen
playloop:       in      al, dx
		test    al, 20h
		jnz     playloop        ; solange bis gedrueckt
		print   ok              ; dann "ok" ausgeben
playok:         call    motoron         ; Motor einschalten
		ret
playkey         endp

; ===========================================================================
; Prozedur srchmsg gibt aus, wonach gesucht wird.
; ===========================================================================

srchmsg         proc    near
		print   searching       ; "searching " ausgeben
		mov     si, tapename    ; Name des gesuchten Programmes
		cmp     si, 0           ; falls kein Programmname
		je      smnoname        ; dann auch nichts ausgeben
		mov     cx, tnlen       ; Laenge des Programmnamen
		call    sprint          ; Programmnamen ausgeben
smnoname:       print   eol             ; Zeilenende ausgeben
		ret
srchmsg         endp

; ===========================================================================
; Prozedur prephead bearbeitet den Header. Es wird der Dateiname in ASCII-
; Darstellung umgewandelt und dabei seine Laenge ermittelt.
; ===========================================================================

prephead        proc    near
; MZ-80B hat noraml ASCII
;		lea     bx, shasctab            ; Umwandlungstabelle
 		mov     si, 0                   ; erstes Zeichen im Namen
phloop:         mov     al, filename[si]        ; Zeichen holen
		cmp     al, 0dh                 ; falls CR:
		je      phexit                  ; dann Ende gefunden
;		xlatb                           ; sonst Zeichen uebersetzen
 		mov     fname[si], al           ; und ablegen
		inc     si                      ; naechstes Zeichen
		cmp     si, 17                  ; falls nicht Ende Namenber.
		jb      phloop                  ; dann weiter uebersetzen
		jmp     phstlen                 ; sonst fertig
phexit:         xlatb                           ; Endekennung uebersetzen
		mov     fname[si], al           ; und ablegen
phstlen:        mov     fnlen, si               ; Namenlaenge ablegen
		ret
prephead        endp

; ===========================================================================
; Prozedur foundmsg gibt eine Meldung ueber die gefundene Cassettendatei aus.
; ===========================================================================

foundmsg        proc    near
		print   found           ; "found " ausgeben
		lea     si, fname       ; umgewandelten Namen
		mov     cx, fnlen
		call    sprint          ; ausgeben
		print   eol             ; Zeilenende ausgeben
		ret
foundmsg        endp

; ===========================================================================
; Prozedur readhead liest einen Header ein und gibt die entsprechenden
; Meldungen aus.
; ===========================================================================

readhead        proc    near
hdloop:         cli                     ; Interrupts sperren
		call    initcnt         ; Zaehler initialisieren
		mov     bx, 40          ; Gap und langen Tapemark suchen
		call    readgap
		lea     di, attrib      ; Anfangsadresse des Headers
		mov     cx, 128         ; Laenge des Headers
		call    readblk         ; Header einlesen
		jnc     hdfstok         ; kein Fehler, dann verzweigen
		call    sync            ; neu synchronisieren
		lea     di, attrib      ; Anfangsadresse des Headers
		mov     cx, 128         ; Laenge des Headers
		call    plainblk        ; Header einlesen
		sti                     ; Interrupts wieder zulassen
		jnc     hdsndok         ; kein Fehler, dann verzweigen
		print   hdcorrupt       ; Fehlermeldung "korrupter Header"
		jmp     hdloop          ; neuen Header suchen
hdsndok:        call    prephead        ; Header vorbereiten
		call    foundmsg        ; Meldung "found ..." ausgeben
		print   hdusesnd        ; Meldung ueber kaputte erste Kopie
		ret
hdfstok:        cmp     single, 0       ; falls Single-Option nicht angegeben
		je      hdverify        ; dann verzweigen
		sti                     ; sonst Interrupts wieder zulassen
		jmp     hdverok         ; und erfolgreich beenden
hdverify:       call    sync            ; neu synchronisieren
		lea     di, attrib      ; Anfangsadresse des Headers
		mov     cx, 128         ; Laenge des Headers
		call    verify          ; mit Kopie vergleichen
		sti                     ; Interrupts wieder zulassen
		jnc     hdverok         ; kein Fehler, dann verzweigen
		call    prephead        ; Header vorbereiten
		call    foundmsg        ; Meldung "found ..." ausgeben
		print   hdvererr        ; Verifikationsfehler melden
		ret
hdverok:        call    prephead        ; Header vorbereiten
		call    foundmsg        ; Meldung "found ..." ausgeben
		ret
readhead        endp

; ===========================================================================
; Prozedur compare vergleicht den Namen des gefundenen Programmes mit dem
; Namen des gesuchten Programmes und gibt bei Ungleichheit ein gesetztes
; Carry-Flag zurueck, bei Gleichheit ein geloeschtes. Wurde kein zu suchender
; Programmname in der Kommandozeile angegeben, so wird ein geloeschtes
; Carry-Flag zurueckgegeben und damit angezeigt, dass das gefundene Programm
; geladen werden soll.
; ===========================================================================

compare         proc    near
		mov     si, tapename    ; Name des gesuchten Programmes
		cmp     si, 0           ; falls nicht angegeben
		je      compeq          ; dann Namen als gleich ansehen
		mov     cx, tnlen       ; Laenge des gesuchten Namen
		cmp     cx, fnlen       ; falls kuerzer als gefundener Name
		jb      compneq         ; dann sind die Namen ungleich
		cmp     cx, 0           ; falls beide Namen leer
		je      compeq          ; dann sind sie gleich
		lea     di, fname       ; Name des gefundenen Programmes
		cld                     ; in Aufwaertsrichtung vergleichen
		repe    cmpsb           ; Namen vergleichen
		jne     compneq         ; bei Ungleichheit verzweigen
compeq:         clc                     ; Namensgleichheit melden
		ret
compneq:        stc                     ; Namensungleichheit melden
		ret
compare         endp

; ===========================================================================
; Prozedur readdata liest den naechsten Datenblock ein und gibt die ent-
; sprechenden Meldungen dazu aus. Ein gesetztes Carryflag zeigt an, dass
; die Daten nicht geladen werden konnten.
; ===========================================================================

readdata        proc    near
		cli                     ; Interrupts sperren
		call    initcnt         ; Zaehler initialisieren
		mov     bx, 20          ; Gap und kurzen Tapemark suchen
		call    readgap
		lea     di, data        ; Anfang des Datenbereichs
		mov     cx, datalen     ; Datenlaenge
		call    readblk         ; Daten einlesen
		jnc     datfstok        ; kein Fehler, dann verzweigen
		call    sync            ; neu synchronisieren
		lea     di, data        ; Anfang der Datenbereichs
		mov     cx, datalen     ; Datenlaenge
		call    plainblk        ; Daten einlesen
		sti                     ; Interrupts wieder zulassen
		jnc     datsndok        ; kein Fehler, dann verzweigen
		print   datcorrupt      ; Fehlermeldung "korrupte Daten"
		stc                     ; Fehlerkennung setzen
		ret
datsndok:       print   datusesnd       ; Meldung ueber kaputte erste Kopie
		jmp     datok           ; erfolgreich beenden
datfstok:       cmp     single, 0       ; falls Single-Option nicht angegeben
		je      datverify       ; dann verzweigen
		sti                     ; sonst Interrupts wieder zulassen
		jmp     datok           ; und mit Erfolg beenden
datverify:      call    sync            ; neu synchronisieren
		lea     di, data        ; Anfang des Datenbereichs
		mov     cx, datalen     ; Datenlaenge
		call    verify          ; mit Kopie vergleichen
		sti                     ; Interrupts wieder zulassen
		jnc     datok           ; kein Fehler, dann verzweigen
		print   datvererr       ; sonst Verifikationsfehler melden
datok:          clc                     ; Erfolg melden
		ret
readdata        endp

; ===========================================================================
; Prozedur fexists gibt ein gesetztes Carry-Flag zurueck, wenn die Datei
; mit dem in dname abgelegten Namen bereits existiert oder wenn beim
; Oeffnen der Datei irgendein Fehler aufgetreten ist.
; ===========================================================================

fexists         proc    near
		mov     ah, 3dh         ; Datei oeffnen
		mov     al, 00h         ; Dateiattribut 'nur lesen'
		lea     dx, dname       ; Dateiname
		int     21h             ; DOS aufrufen
		jnc     fclose          ; evtl. Datei wieder schliessen
		cmp     ax, 2           ; Fehlermeldung 2 = Datei ex. nicht
		je      fnex
		jmp     fex             ; anderer Fehler, dann Carry setzen
fclose:         mov     bx, ax          ; Dateihandle
		mov     ah, 3eh         ; Datei schliessen
		int     21h             ; DOS aufrufen
fex:            stc                     ; Datei existiert
		ret
fnex:           clc                     ; Datei existiert nicht
		ret
fexists         endp

; ===========================================================================
; Prozedur prepdname bereitet den DOS-Namen vor, falls keiner in der
; Kommandozeile angegeben wurde. Hierzu wird aus dem Namen der gefundenen
; Datei ein den DOS-Konventionen entsprechender Dateiname mit der Endung
; .M12 erzeugt. Ist bereits eine Datei mit diesem Namen vorhanden, so wird
; die Endung in .M13, .M14, .M15 usw. umbenannt, bis ein freier Name gefun-
; den wird. Kann kein freier Name gefunden werden, so wird dies durch ein
; gesetztes Carry-Flag angezeigt.
; ===========================================================================

prepdname       proc    near
		mov     ax, dosname     ; falls DOS-Name angegeben
		cmp     ax, 0
		jne     pdready         ; dann schon fertig
		lea     si, fname       ; gefundener Name
		mov     cx, fnlen       ; Laenge des gefundenen Namen
		lea     di, dname       ; zu konstruierender Name
		mov     dosname, di     ; gleich neuer DOS-Name
pdnloop:        cmp     cx, 0           ; falls gefundener Name fertig
		je      pdnexit                 ; dann raus aus der Schleife
		cmp     di, offset dname+8      ; falls DOS-Name am Ende
		jae     pdnexit                 ; dann raus aus der Schleife
		mov     al, [si]        ; sonst Zeichen holen
		cmp     al, "0"         ; Zeichen "0".."9" uebernehmen
		jb      pdnnext
		cmp     al, "9"
		jbe     pdntake
		cmp     al, "A"         ; Zeichen "A".."Z" uebernehmen
		jb      pdnnext
		cmp     al, "Z"
		jbe     pdntake
		sub     al, "a"-"A"     ; in Grossbuchstaben umwandeln
		cmp     al, "A"         ; Zeichen "A".."Z" uebernehmen
		jb      pdnnext
		cmp     al, "Z"
		jbe     pdntake
		jmp     pdnnext         ; alle anderen Zeichen verwerfen
pdntake:        mov     [di], al        ; gutes Zeichen in DOS-Namen nehmen
		inc     di              ; naechste Position im DOS-Namen
pdnnext:        inc     si              ; neue Position in gefundenem Namen
		dec     cx              ; Zaehler fuer Namenlaenge erniedr.
		jmp     pdnloop                 ; zurueck zum Schleifenanfang
pdnexit:        cmp     di, offset dname        ; falls Name nicht leer
		jne     pdnamok                 ; dann ok
		lea     si, defname     ; sonst Defaultnamen nehmen
		mov     cx, defnlen     ; Laenge des Defaultnamen
		rep     movsb           ; in den Namenbereich kopieren
pdnamok:        lea     si, defext      ; Defaultextension
		mov     cx, defxlen     ; Laenge der Defaultextension
		rep     movsb                   ; an den Namen anhaengen
		mov     byte ptr [di], 00h      ; Namen mit Nullbyte abschl.
		mov     ax, di                  ; Laenge des Namen berechnen
		sub     ax, offset dname
		mov     dnlen, ax               ; und ablegen
pdexloop:       call    fexists                 ; pruefen ob Name schon vorh.
		jc      pdincex                 ; ja, dann Extension modif.
pdready:        clc                             ; Erfolg melden
		ret                             ; und fertig
pdincex:        inc     byte ptr [di-1]         ; sonst Extension erhoehen
		cmp     byte ptr [di-1], "9"    ; zunaechst letzte Ziffer
		jbe     pdexloop                ; falls Uebertrag
		mov     byte ptr [di-1], "0"    ; dann vorletzte Ziffer erh.
		inc     byte ptr [di-2]
		cmp     byte ptr [di-2], "9"
		jbe     pdexloop                ; falls Uebertrag
		mov     byte ptr [di-2], 0      ; dann erstes Ext.zchn erh.
		inc     byte ptr [di-3]
		cmp     byte ptr [di-3], "Z"
		jbe     pdexloop                ; falls immer noch Uebertrag
		stc                             ; dann Fehler melden
		ret
prepdname       endp

; ===========================================================================
; Prozedur savemsg gibt eine Meldung darueber aus, unter welchem Namen die
; vom Band geladene Datei auf die Platte geschrieben wird.
; ===========================================================================

savemsg         proc    near
		print   saving          ; "saving" ausgeben
		mov     si, dosname
		mov     cx, dnlen
		call    sprint          ; Dateinamen ausgeben
		print   eol             ; Zeilenende ausgeben
		ret
savemsg         endp

; ===========================================================================
; Prozedur save speichert die geladene Datei ab. Beim Auftreten eines Fehlers
; wird das gesetzte Carry-Flag zurueckgegeben.
; ===========================================================================

save            proc    near
		mov     ah, 3ch         ; neue Datei erstellen
		mov     cx, 0           ; Dateiattribut normal
		mov     dx, dosname     ; Dateiname
		int     21h             ; DOS aufrufen
		jc      savend          ; eventuell Fehler aufgetreten
		mov     bx, ax          ; Dateihandle
		mov     ah, 40h         ; Datei beschreiben
		mov     cx, datalen     ; Anzahl der zu schreibenden Bytes
		add     cx, 128         ; Header beruecksichtigen
		lea     dx, attrib      ; Anfang der zu schreibenden Daten
		int     21h             ; DOS aufrufen
		jc      saverr          ; eventuell Fehler aufgetreten
		cmp     ax, cx          ; falls Anzahl geschriebener Bytes
		jne     saverr          ; nicht korrekt, dann Fehler
		mov     ah, 3eh         ; Datei schliessen
		int     21h             ; DOS aufrufen
savend:         ret                     ; evtl. Fehler wird gemeldet
saverr:         mov     ah, 3eh         ; Datei schliessen
		int     21h             ; DOS aufrufen
		stc                     ; Fehler melden
		ret
save            endp

; ===========================================================================
; Hauptprogramm
; ===========================================================================

mainprog:       call    prepargs        ; Kommandozeile bearbeiten
		jnc     argsok          ; falls Kommandozeilenfehler
		print   info            ; Informationen ausgeben
errexit:        mov     ax, 4c01h       ; Programm mit Fehler beenden
		int     21h
argsok:         call    playkey         ; PLAY-Taste druecken
		call    srchmsg         ; Suchmeldung ausgeben
srchloop:       call    readhead        ; Header einlesen
		call    compare         ; Namen vergleichen
		jc      srchloop        ; bei Ungleichheit weitersuchen
		mov     ax, sp                  ; verfuegbaren Speicher
		sub     ax, offset data+100h    ; ermitteln
		cmp     ax, datalen             ; mit Programmlaenge vergl.
		jae     doload                  ; genug Speicher, dann laden
		print   toobig                  ; sonst Fehler melden
moerrexit:      call    motoroff                ; Motor ausschalten
		jmp     errexit                 ; und Programm beenden
doload:         print   loading         ; "loading" ausgeben
		call    readdata        ; Programmdaten einlesen
		jnc     ldok            ; falls Ladefehler
		print   lderror         ; "load error" ausgeben
		jmp     moerrexit       ; Programm mit Fehler beenden
ldok:           call    motoroff        ; Motor ausschalten
		call    prepdname       ; DOS-Namen vorbereiten, falls noetig
		jnc     psavmsg         ; falls kein Name zu finden ist
		print   noname          ; dann Meldung darueber ausgeben
		jmp     errexit         ; und Programm beenden
psavmsg:        call    savemsg         ; sonst Meldung "saving ..." ausgeben
		call    save            ; Programm abspeichern
		jnc     svok            ; falls Schreibfehler
		print   sverror         ; "disk write error" ausgeben
		jmp     errexit         ; Programm mit Fehler beenden
svok:           print   ready           ; "ready" ausgeben
		mov     ax, 4c00h       ; Programm erfolgreich beenden
		int     21h

; ===========================================================================
; Uninitialisierter Datenbereich
; ===========================================================================

		align   word            ; allgemeine Variablen

dosname         dw      ?               ; Name der DOS-Datei (Adresse)
dnlen           dw      ?               ; Laenge des Namen
tapename        dw      ?               ; Name der Cassettendatei (Adresse)
tnlen           dw      ?               ; Laenge des Namen
fnlen           dw      ?               ; Laenge des gefundenen Namen
diff            dw      ?               ; Anzahl Verifikationsabweichungen
inverse         db      ?               ; <>0 = Polaritaet invertieren
single          db      ?               ; <>0 = Kopie nicht lesen
fname           db      17 dup (?)      ; gefundener Dateiname
dname           db      12 dup (?)      ; neu konstruierter DOS-Name

		align   word            ; Header und Daten

attrib          db      ?               ; Dateiattribut
filename        db      17 dup (?)      ; Dateiname
datalen         dw      ?               ; Dateilaenge
loadadr         dw      ?               ; Ladeadresse
startadr        dw      ?               ; Ausfuehrungsadresse
comments        db      104 dup (?)     ; Kommentar
data            db      ?

; ===========================================================================
; Ende des Programmsegmentes
; ===========================================================================

main            ends
		end     start
